import requests
from bs4 import BeautifulSoup
import pandas as pd

def p2f(x):
    return float(x.strip('%'))/100

def create_prob_pickle(teams, scoring, outputname, mapping_path):
    """
    teams: int, number of teams in league
    outputname: string, filename for the output probability pickle
    mapping_path: string, filepath for the mapping file. 
    scoring: string, valid entries are "standard", "ppr", "half-ppr", or "2qb"
    """
    
    
    score=scoring
    
    url = r"https://fantasyfootballcalculator.com/scenario-calculator"
    params = lambda x,teams,scoring: {'format':scoring,
              'num_teams':teams,
              'draft_pick':x}

    probabilities = {}
    html = {}
    print('1 of 4: scraping picks - this will take a minute')
    for i in range(1,201):
        test = requests.get(url,params(i,teams,score))
        if test.status_code == 200:
            soup = BeautifulSoup(test.text, 'html.parser')
            html[i] = test
            probabilities[i] = soup.select("table")
        else:
            print(f"{i} failed")
            print(test.status_code)

    df = pd.read_html(str(soup.find_all('table')[0]))
    print('2 of 4: scraping done - starting processing')
    dfs = None
    for k, table in probabilities.items():
        df = pd.read_html(str(table[0]))[0]
        df['pick'] = k
        if dfs is None:
            dfs = df
        else:
            dfs = dfs.append(df)
    
    mapping = pd.read_csv('probmap.csv')
    dfs = dfs.merge(mapping, how='left', left_on='Name', right_on='probname')
    dfs['%'] = dfs['%'].apply(p2f)
    dfs['espnid'] = dfs['espnid'].astype(str)
    dfs[['Name','Pos','Team','Bye','%','pick','espnid']].to_pickle(outputname)
    print(f'3 of 4: probabilities updated')
    print(f'4 of 4: pickle saved as {outputname}')
    pass





import pandas as pd
import numpy as np

oc_mult = dict(zip(['QB','RB','WR','TE','K','D/ST'],[2/3, 1/4, 1/4, 2/3, 1, 1]))

def process_projections(projection_path,idmap_path,output_path):
    print("1 of 4: reading projections")
    proj = pd.read_csv(projection_path)
    proj = proj[proj['position'].isin(['RB','QB','WR','TE','K','DST'])]

    print("2 of 4: reading id mapping")
    espnidmap = pd.read_csv(idmap_path)

    proj = proj.merge(espnidmap, how='left', on='player')
    proj = proj[proj['rank'] <= 250]
    proj = proj.drop_duplicates(subset=['player','team','position'])
    proj = proj[proj['rank'] <= 250]
    proj = proj.sort_values('points', ascending=False)
    proj['picked'] = False
    proj['blacklist'] = False
    proj['espn_id'] = proj['espn_id'].astype(str)
    proj['oc_mult'] = proj['position'].map(oc_mult)
    proj.index = proj['espn_id']
    proj.index.name = None
    id_map = pd.read_excel('broad_id_map.xlsx', dtype=str)
    proj = proj.merge(id_map, how='left', on='espn_id')
    proj.index = proj['espn_id']
    proj.index.name = None
    print("3 of 4: saving pickle")
    proj.to_pickle(output_path)
    print("4 of 4: done")
    pass








import yahoowizard.draft_logic as dl
from .update_probabilities import *
from .process_and_map_projections import *
from .yahoo_draft import *

def tell_me_what_to_do(players,prob,draft):
    players = draft.update(players)
    np1, np2 = dl.next_picks(draft.pick_order,draft.my_team, draft.current_pick)
    px = dl.adj_probs(players,prob,np1,np2)
    print(f'this pick is {draft.current_pick}')
    print(f'Your Next Pick is {np1}')
    print(f'Your 2nd Pick is {np2}')
    return px

def unblacklist_player(name,team,df):
    name_match = df['player'] == name
    team_match = df['team'] == team
    mask = name_match & team_match
    print("Removing these players from the blacklist:")
    print(df[mask][['player','team','position']])
    df['blacklist'] = np.where(mask, False, df['blacklist'])
    return df

def blacklist_player(name,team,df):
    name_match = df['player'] == name
    team_match = df['team'] == team
    mask = name_match & team_match
    print("Blacklisting these players:")
    print(df[mask][['player','team','position']])
    df['blacklist'] = np.where(mask, True, df['blacklist'])
    return df

def blacklist_position(position):
    "positions can be: 'QB','RB','WR','TE','DST','K'"
    df['blacklist'] = np.where(df['position'] == position, True, df['blacklist'])
    return df

def unblacklist_position(position):
    "positions can be: 'QB','RB','WR','TE','DST','K'"
    df['blacklist'] = np.where(df['position'] == position, False, df['blacklist'])


import pandas as pd
import numpy as np


def next_picks(pick_order, mypick, thispick):
    next_pick = pick_order.index(mypick,thispick) + 1
    next_pick2 = pick_order.index(mypick,next_pick) + 1
    return next_pick, next_pick2

def p2f(x):
    return float(x.strip('%'))/100

def expected_max(df):
    df['pbest'] = df['probpicked'].cumprod().shift().fillna(1)
    df['emaxw'] = df['pbest'] * df['%']
    df['emax'] = df['points'] * df['emaxw']
    return df

def adj_probs(players,probs, next_pick, next_pick2):
    # players['oc_adj'] = players['need_adj']*1 + ~players['need_adj']*players['oc_mult']
    players['penalty'] = np.where(players['need_adj'], players['oc_mult'] * players['bench_count'], 0)
    players['penalty'] = np.clip(players['penalty'], 0, 1)
    players['oc_adj'] = 1 - players['penalty']
    players['espn_id'] = players['espn_id'].astype(str)
    probs['espnid'] = probs['espnid'].astype(str)
    for i, pick in enumerate([next_pick,next_pick2]):
        pmask = probs['pick'] == pick
        df = players.merge(probs[pmask], how='left', left_on='espn_id', right_on='espnid')
        df = df.drop_duplicates(subset=['player','team'])
        df = df.drop_duplicates(subset=['espn_id'])
        df.index = df['espn_id']
        df.index.name = None
        df['%'] = df['%'].fillna(1)
        df['%'] = np.where(df['picked'] | df['blacklist'], 0, df['%'])
        df['probpicked'] = 1 - df['%']
        df = df.groupby('position').apply(expected_max)
        df.index = df['espn_id']
        df.index.name = None
        palt = df.groupby('position')['emax'].sum()
        df['opp'] = df['position'].map(palt)
        players[f'oc_raw_{i}'] = (df['points'] - df['opp']) * ~df['picked'] * ~df['blacklist']
        players[f'pb_{i}'] = df['probpicked']
        players[f'oc_adj_{i}'] = players[f'oc_raw_{i}'] * players['oc_adj'] * players[f'pb_{i}']
    return players.copy()





import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="yahoo-draft-wizard", # Replace with your own username
    version="0.0.7",
    author="James Whiting",
    author_email="jlwhiting@outlook.com",
    description="Dynamically recommends picks for your yahoo fantasy football draft",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/jwaggie14/yahoo-draft-wizard",
    install_requires=[
        'numpy',
        'pandas',
        'yahoo_oauth'],
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
)



import xml.etree.ElementTree as ET 
import re
import pandas as pd
import numpy as np
from yahoo_oauth import OAuth2

ns = {'xlmns': 'http://fantasysports.yahooapis.com/fantasy/v2/base.rng',
      'yahoo': "http://www.yahooapis.com/v1/base.rng"}

oauth = OAuth2(None, None, from_file='oauth.json')
if not oauth.token_is_valid():
    oauth.refresh_access_token()

oc_mult = dict(zip(['QB','RB','WR','TE','K','DST'],[.3,.5,.5,.3,.15,.15]))

flex_positions = ['RB','WR','TE']
    
    
    
class YahooDraft:
    def __init__(self, league_id, my_first_pick):
        self.league_id = str(league_id)
        self.first_pick = my_first_pick
        self.my_team = my_first_pick
        self.league_url = f"https://fantasysports.yahooapis.com/fantasy/v2/league/nfl.l.{self.league_id}"
        self.get_settings()
        pass
    
    
    
    def get_settings(self):
        url = f"{self.league_url}/settings"
        r = oauth.session.get(url)
        xmlstring = r.text
        xmlstring = re.sub(' xmlns="[^"]+"', '', xmlstring, count=1)
        root = ET.fromstring(xmlstring)
        self.num_teams = int(root[0].find('num_teams').text)

        positions = {}
        settings = root[0].find('settings')
        rp = settings.find('roster_positions')
        
        yahoo_pos_map = {
            'QB':'QB',
            'WR':'WR',
            'RB':'RB',
            'TE':'TE',
            'W/R/T':'FLEX',
            'DEF':'DST',
            'K':'K',
            'BN':'BEN',
            'IR':'BEN'
        }
        
        for c in rp:
            ypos = yahoo_pos_map[c[0].text]
            if ypos == 'BEN':
                positions[ypos] = int(c[1].text)
            else:
                positions[ypos] = int(c[2].text)
                
        self.req_positions = pd.Series(positions)
        
        self.rounds = self.req_positions.sum()
        self.pick_order = (list(range(1,self.num_teams+1)) + list(range(self.num_teams,0,-1))) * (self.rounds // 2)
        if self.rounds % 2 == 1:
            self.pick_order += list(range(1,self.num_teams+1))
        self.current_pick = 1
        pass
    
    
    
    def update(self, projections):
        self.scrape_draft()
        self.slot_needs(projections)
        projections = self.filter_players(projections)
        projections['need_adj'] = projections['position'].map(self.pos_needs['need'])
        projections['bench_count'] = projections['position'].map(self.slot_requirements)
        projections['bench_count'] = np.clip(-projections['bench_count'], 0, 10)
        return projections
        
    
    
    def scrape_draft(self):
        players = {}
        teams = self.num_teams
        for i in range(1,teams+1):
            _url = f"{self.league_url}/teams;team_keys=nfl.l.{self.league_id}.t.{i}/players"
            _r = oauth.session.get(_url)
            xmlstring = _r.text
            xmlstring = re.sub(' xmlns="[^"]+"', '', xmlstring, count=1)
            root = ET.fromstring(xmlstring)
            for player in root.iter('player_id'):
                players[player.text] = i
        self.picked_players = pd.Series(players, name='team')
        self.team_mask = self.picked_players == self.my_team
        self.team_roster = self.picked_players[self.team_mask]
        self.current_pick = self.picked_players.shape[0] + 1
        pass

    
    
    
    def slot_needs(self, projections):
        filled_slots = projections[projections['yahoo_id'].isin(self.team_roster.index)]['position'].value_counts()
        open_slots = self.req_positions.subtract(filled_slots, fill_value=0)
        
        needs = open_slots[['QB','RB','WR','TE','FLEX','K','DST']]
        flex_need = needs[flex_positions].min() >= 0

        pos_needs = pd.DataFrame(open_slots > 0, columns=['slot'])
        pos_needs['flex'] = np.where(pos_needs.index.isin(flex_positions), flex_need, False)
        pos_needs['need'] = pos_needs['slot'] | pos_needs['flex']
        self.pos_needs = pos_needs
        self.slot_requirements =  needs
        pass
    
    
    
    def filter_players(self, projections):
        projections['picked'] = projections['yahoo_id'].isin(self.picked_players.index)
        return projections
